#!/usr/bin/env python3

import os
import sys
import zipfile

from flask import Flask, render_template, send_file, request

from . import file
from . import config

app = Flask(
    __name__,
    template_folder="../templates",
    static_folder="../static",
)

app.jinja_env.globals.update(
    config=config,
    path_join=os.path.join,
)


@app.route('/', defaults={'path': ''})
@app.route('/<path:path>')
def index(path):
    internal_path = os.path.join(config.get('base_path'), path)
    path = '/' + path

    # Checks ###################################################

    if '..' in path:
        return 'Path cannot contain double dots, i.e. "..".'

    if not os.path.exists(internal_path):
        return (
            render_template(
                "error.html",
                code=404,
                msg=f'The path "{path}" does not exist.',
            ),
            404,
        )

    # Actual serving ###########################################

    if os.path.isdir(internal_path):
        _, dirs, files = next(os.walk(internal_path))
        files = sorted(filter(lambda x: not x.startswith("."), files))
        dirs = sorted(filter(lambda x: not x.startswith("."), dirs))

        return (
            render_template(
                "directory.html",
                path=path,
                dirs=dirs,
                files=files,
            ),
            200,
        )

    # Path exists, not a folder, must be a file, send
    return send_file(internal_path)


@app.route('/mass_download')
def mass_download():
    internal_base = config.get('base_path')

    base = request.args.get('base', '')
    files = request.args.getlist('file')

    # Checks ###################################################

    if base == '':
        return {"error": "Base path not supplied"}, 400

    if '..' in base or base[0] != '/':
        return {"error": "Invalid base path"}, 400

    # TODO: If it's 1 file and it's not a directory, then redirect to the file
    if len(files) < 1:
        return {"error": "A minimum of two files is required"}, 400

    # TODO: Check if the files exist
    # TODO: Check if the files are bigger than a certain amount?

    # Actual serving ###########################################

    for i_file in files:
        internal_path = os.path.join(internal_base, base[1:], i_file)

        # FIXME: I'm broke
        if os.path.isdir(internal_path):
            for root, _, subfiles in os.walk(internal_path):
                for i_subfile in subfiles:
                    external_root = root[len(internal_base) + 1:]
                    print(external_root)
                    files.append(os.path.join(root, external_root))
                print(files)

    with zipfile.ZipFile('/tmp/download.zip', 'w') as f_zip:
        for i_file in files:
            internal_path = os.path.join(internal_base, base[1:], i_file)

            # print(internal_path)

            if not os.path.exists(internal_path):
                # And what is this?¿
                return {'error': f'The following path does not exist: "{i_file}"'}, 400

            f_zip.write(internal_path)

    return [base, files], 200


@app.route('/search')
def search():
    q = request.args.get('q', '')

    if q == '':
        return (
            render_template(
                "error.html",
                code=400,
                msg='No search string provided.',
            ),
            400,
        )

    ret = file.search(q)

    return (
        render_template(
            "search.html",
            list=ret,
            q=q,
        ),
        200,
    )
